@@ -435,6 +435,14 @@ class Transactional(Dotkeys):
         else:
             Dotkeys.__delitem__(self, key)
 
+    def option(self, key, value):
+        self[key] = value
+        return self
+
+    def unset(self, key):
+        del self[key]
+        return self
+
     def set_item(self, key, value):
         with self._direct_state:
             self[key] = value
@@ -697,7 +705,9 @@ class Interface(Transactional):
                     # Operation not supported
                     if x.code == 95 and request.get('index', 0) != 0:
                         # ACHTUNG: hack for old platforms
-                        request['index'] = 0
+                        request = IPLinkRequest({'ifname': self['ifname'],
+                                                 'kind': self['kind'],
+                                                 'index': 0})
                         compat.fix_create_link(self.nl, request)
                     else:
                         raise
@@ -720,8 +730,7 @@ class Interface(Transactional):
 
             # all is OK till now, so continue
             # we do not know what to load, so load everything
-            self.nl.get_links()
-            self.ipdb._links_event.wait(_SYNC_TIMEOUT)
+            self.ipdb.wait_interface(ifname=self['ifname'])
 
         # now we have our index and IP set and all other stuff
         snapshot = self.pick()
@@ -889,14 +898,20 @@ class Interface(Transactional):
         '''
         Shortcut: change the interface state to 'up'.
         '''
-        self['flags'] |= 1
+        if self['flags'] is None:
+            self['flags'] = 1
+        else:
+            self['flags'] |= 1
         return self
 
     def down(self):
         '''
         Shortcut: change the interface state to 'down'.
         '''
-        self['flags'] &= ~(self['flags'] & 1)
+        if self['flags'] is None:
+            self['flags'] = 0
+        else:
+            self['flags'] &= ~(self['flags'] & 1)
         return self
 
     def remove(self):
@@ -1147,6 +1162,12 @@ class IPDB(object):
         self._links_event = threading.Event()
         self.exclusive = threading.RLock()
 
+        # we have to move it here 'cause of stupid
+        # python bug in RHEL6.5, that is yet to be
+        # investigated
+        self.nl.monitor()
+        self.nl.mirror()
+
         # load information on startup
         links = self.nl.get_links()
         for link in links:
@@ -1158,8 +1179,6 @@ class IPDB(object):
         self.update_routes(routes)
 
         # start monitoring thread
-        self.nl.monitor()
-        self.nl.mirror()
         self._mthread = threading.Thread(target=self.serve_forever)
         self._mthread.setDaemon(True)
         self._mthread.start()
@@ -1277,10 +1296,12 @@ class IPDB(object):
                 self.by_name[ifname] = \
                 self.interfaces[ifname] = \
                 self.iclass(ipdb=self, mode='snapshot')
+            device.update(kwarg)
+            if isinstance(kwarg.get('link', None), Interface):
+                device['link'] = kwarg['link']['index']
             device['kind'] = kind
             device['index'] = kwarg.get('index', 0)
             device['ifname'] = ifname
-            device.update(kwarg)
             device._mode = self.mode
         device.begin()
         return device
@@ -1365,7 +1386,11 @@ class IPDB(object):
         def cb(self, msg, _action):
             if _action != action:
                 return
-            port = self.interfaces[msg['index']]
+            try:
+                port = self.interfaces[msg['index']]
+            except KeyError:
+                return
+
             for key in kwarg:
                 if port.get(key, None) != kwarg[key]:
                     return
@@ -1442,8 +1467,8 @@ class IPDB(object):
         for addr in addrs:
             nla = get_addr_nla(addr)
             if nla is not None:
-                method = getattr(self.ipaddr[addr['index']], action)
                 try:
+                    method = getattr(self.ipaddr[addr['index']], action)
                     method(key=(nla, addr['prefixlen']), raw=addr)
                 except:
                     pass
